!+
! Program:  epchain/test/checkoutput_BU
!
! Authors:  Michael Freyberg - XMM-SSC-MPE
!          
! Description: Tests output and functionality of epchain
!              for BURST mode
!- 


   program checkoutput_BU

   use dal
   use errorhandling

   implicit none

   type(DataSetT)  :: inevents
   type(TableT)    :: eventstable, exptable, bpixtable

   character(len=8)                          :: extname					     
					     
   integer nrows

   call message(AppMsg,Noisy,"testing BURST mode epchain output:")

   !-----------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing that all expected extensions are present...")
   !-----------------------------------------------------------------

   inevents = dataSet("P0000000000PNS004TIEVLI0000.FIT", read)

   if (.not.hasBlock(inevents,"EVENTS")) &
      call error("noEvExt","Events extension absent or incorrectly named")

   write(extname,'(a)') "EXPOSU04"
   if (.not.hasBlock(inevents,extname)) then
     write(errstr,*) "Exposure extension ",extname," absent or incorrectly named"
     call error("noExpExt")
   endif

   write(extname,'(a)') "BADPIX04"
   if (.not.hasBlock(inevents,extname)) then
      write(errstr,*) "Badpix extension ",extname," absent or incorrectly named"
      call error("noBadExt")
   endif

   write(extname,'(a)') "STDGTI04"
   if (.not.hasBlock(inevents,extname)) then
      write(errstr,*) "GTI extension ",extname," absent or incorrectly named"
      call error("noGtiExt")
   endif

   !----------------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing existence of standard primary header keywords...")
   !----------------------------------------------------------------------
   if (.not.hasAttribute(inevents,"TELESCOP") .or. &
       .not.hasAttribute(inevents,"INSTRUME") .or. &
       .not.hasAttribute(inevents,"DATAMODE") .or. &
       .not.hasAttribute(inevents,"FILTER")   .or. &
       .not.hasAttribute(inevents,"OBS_MODE") .or. &
       .not.hasAttribute(inevents,"OBS_ID")   .or. &
       .not.hasAttribute(inevents,"EXP_ID")   .or. &
       .not.hasAttribute(inevents,"CONTENT")  .or. &
       .not.hasAttribute(inevents,"ORIGIN")   .or. &
       .not.hasAttribute(inevents,"OBJECT")   .or. &
       .not.hasAttribute(inevents,"RA_PNT")   .or. &
       .not.hasAttribute(inevents,"DEC_PNT")  .or. &
       .not.hasAttribute(inevents,"PA_PNT")   .or. &
       .not.hasAttribute(inevents,"RADECSYS") .or. & 
       .not.hasAttribute(inevents,"EQUINOX")  .or. &
!      .not.hasAttribute(inevents,"EXPOSURE") .or. &
       .not.hasAttribute(inevents,"DATE-OBS") .or. &
       .not.hasAttribute(inevents,"DATE-END") .or. &
       .not.hasAttribute(inevents,"OBSERVER") .or. &
       .not.hasAttribute(inevents,"RA_OBJ")   .or. &
       .not.hasAttribute(inevents,"DEC_OBJ")  .or. &
       .not.hasAttribute(inevents,"RA_NOM")   .or. &
       .not.hasAttribute(inevents,"DEC_NOM"))      &
       call error("noKeyword","Standard primary header keyword is missing")

   !---------------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing settings of standard primary header keywords...") 
   !---------------------------------------------------------------------
   if (stringAttribute(inevents,"TELESCOP") /= "XMM"      .or. &
    stringAttribute(inevents,"INSTRUME") /= "EPN"         .or. &
    stringAttribute(inevents,"DATAMODE") /= "TIMING"      .or. &
    stringAttribute(inevents,"FILTER") /= "UNKNOWN"       .or. &
    stringAttribute(inevents,"OBS_MODE") /= "POINTING"    .or. &
    stringAttribute(inevents,"CONTENT") /= "EPIC TIMING MODE EVENT LIST".or.&
    stringAttribute(inevents,"ORIGIN") /= "Leicester/SSC" .or. &
    stringAttribute(inevents,"OBJECT") /= "duckling"      .or. &
!    real32Attribute(inevents,"RA_PNT") /= 0               .or. &
!    real32Attribute(inevents,"DEC_PNT") /= 0              .or. &
!    real32Attribute(inevents,"PA_PNT") /= 0               .or. &
    stringAttribute(inevents,"RADECSYS") /= "FK5"         .or. &        
    real32Attribute(inevents,"EQUINOX") /= 2000           .or. &
    stringAttribute(inevents,"OBSERVER") /= "Dr. Pino Paperino") &
    call error("wrongKeyword", &
              "Wrong value in standard primary header keyword")

   !-------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing column existence in events extension...")
   !-------------------------------------------------------------
   eventstable = table(inevents,"EVENTS")
   nrows = numberOfRows(eventstable)

   if (.not.hasColumn(eventstable,"TIME ")   .or. &
       .not.hasColumn(eventstable,"RAWX")    .or. &
       .not.hasColumn(eventstable,"PHA")     .or. &
       .not.hasColumn(eventstable,"PI")      .or. &
       .not.hasColumn(eventstable,"PATTERN") .or. &
       .not.hasColumn(eventstable,"CCDNR"))       &
       call error("noEvColumn", &
                        "Events extension: column absent or incorrectly named")


   !----------------------------------------------------------------
   call message(AppMSg,Noisy, &
               "testing column existence in exposure extension...")
   !----------------------------------------------------------------
   write(extname,'(a)') "EXPOSU04"
   exptable = table(inevents,extname)

   if (.not.hasColumn(exptable,"TIME")       .or. &
!       .not.hasColumn(exptable,"TIMEDEL")    .or. &
        .not.hasAttribute(exptable,"TIMEDEL")    .or. &
       .not.hasColumn(exptable,"FRACEXP")) then
       write(errstr,*) 'Exposure extension ',extname, &
                       ': column absent or incorrectly named' 
       call error("noExposureColumn")
   endif

   if (stringAttribute(exptable,"DATATYPE") /= "BURST.EL" ) & 
     call error("wrongKeyword", &
              "Wrong value in EXPOSURE extension header keyword DATATYPE")

   !----------------------------------------------------------------
   call message(AppMSg,Noisy, &
               "testing column existence in bad pixel extension...")
   !----------------------------------------------------------------
   write(extname,'(a)') "BADPIX04"
   bpixtable = table(inevents,extname)

   if (.not.hasColumn(bpixtable,"RAWX")       .or. &
       .not.hasColumn(bpixtable,"RAWY")       .or. &
       .not.hasColumn(bpixtable,"TYPE")       .or. &
       .not.hasColumn(bpixtable,"YEXTENT")    .or. & 
       .not.hasColumn(bpixtable,"BADFLAG")) then
       write(errstr,*) 'Badpix extension ',extname, &
                       ': column absent or incorrectly named' 
       call error("noBadpixColumn")
   endif   

   !----------------------
   call release(inevents)

   end

