!+
! Program:  epchain/test/checkoutput_IM
!
! Authors:  Matthias Ehle, Michael Freyberg - XMM-SSC-MPE
!          
! Description: Tests output and functionality of epchain
!              for IMAGING mode
!- 


   program checkoutput_IM

   use dal
   use errorhandling

   implicit none

   type(DataSetT)  :: inevents
   type(TableT)    :: eventstable, exptable, bpixtable

   character(len=8)                          :: extname					     

   integer i, nrows

   call message(AppMsg,Noisy,"testing IMAGING mode epchain output:")

   !-----------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing that all expected extensions are present...")
   !-----------------------------------------------------------------

 ! inevents = dataSet("P0000010060PNS001IMEVLI0000.FIT", read) ! scisimdrift
 ! inevents = dataSet("P0000010050PNS001IMEVLI0000.FIT", read) ! scisimmultimode
   inevents = dataSet("P0000010050PNS001PIEVLI0000.FIT", read) ! 

   if (.not.hasBlock(inevents,"EVENTS")) &
      call error("noEvExt","Events extension absent or incorrectly named")

   do i = 7, 8
    write(extname,'(a,i2.2)') "EXPOSU",i

    if (.not.hasBlock(inevents,extname)) then
      write(errstr,*) "Exposure extension ",extname," absent or incorrectly named"
      call error("noExpExt")
    endif
   enddo 

   do i = 7, 8
    write(extname,'(a,i2.2)') "BADPIX",i

    if (.not.hasBlock(inevents,extname)) then
       write(errstr,*) "Badpix extension ",extname," absent or incorrectly named"
       call error("noBadExt")
    endif
   enddo 

   do i = 7, 8
    write(extname,'(a,i2.2)') "STDGTI",i

    if (.not.hasBlock(inevents,extname)) then
       write(errstr,*) "GTI extension ",extname," absent or incorrectly named"
       call error("noGtiExt")
    endif
   enddo 

   !----------------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing existence of standard primary header keywords...")
   !----------------------------------------------------------------------
   if (.not.hasAttribute(inevents,"TELESCOP") .or. &
       .not.hasAttribute(inevents,"INSTRUME") .or. &
       .not.hasAttribute(inevents,"DATAMODE") .or. &
       .not.hasAttribute(inevents,"FILTER")   .or. &
       .not.hasAttribute(inevents,"OBS_MODE") .or. &
       .not.hasAttribute(inevents,"OBS_ID")   .or. &
       .not.hasAttribute(inevents,"EXP_ID")   .or. &
       .not.hasAttribute(inevents,"CONTENT")  .or. &
       .not.hasAttribute(inevents,"ORIGIN")   .or. &
       .not.hasAttribute(inevents,"OBJECT")   .or. &
       .not.hasAttribute(inevents,"RA_PNT")   .or. &
       .not.hasAttribute(inevents,"DEC_PNT")  .or. &
       .not.hasAttribute(inevents,"PA_PNT")   .or. &
       .not.hasAttribute(inevents,"RADECSYS") .or. & 
       .not.hasAttribute(inevents,"EQUINOX")  .or. &
!      .not.hasAttribute(inevents,"EXPOSURE") .or. &
       .not.hasAttribute(inevents,"DATE-OBS") .or. &
       .not.hasAttribute(inevents,"DATE-END") .or. &
       .not.hasAttribute(inevents,"OBSERVER") .or. &
       .not.hasAttribute(inevents,"RA_OBJ")   .or. &
       .not.hasAttribute(inevents,"DEC_OBJ")  .or. &
       .not.hasAttribute(inevents,"RA_NOM")   .or. &
       .not.hasAttribute(inevents,"DEC_NOM"))      &
       call error("noKeyword","Standard primary header keyword is missing")

   !---------------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing settings of standard primary header keywords...") 
   !---------------------------------------------------------------------
   if (stringAttribute(inevents,"TELESCOP") /= "XMM"      .or. &
    stringAttribute(inevents,"INSTRUME") /= "EPN"         .or. &
    stringAttribute(inevents,"DATAMODE") /= "IMAGING"     .or. &
    stringAttribute(inevents,"FILTER") /= "UNKNOWN"       .or. &
    stringAttribute(inevents,"OBS_MODE") /= "POINTING"    .or. &
    stringAttribute(inevents,"CONTENT") /= "EPIC PN IMAGING MODE EVENT LIST".or.&
    stringAttribute(inevents,"ORIGIN") /= "Leicester/SSC" .or. &
    stringAttribute(inevents,"OBJECT") /= "duckling"      .or. &
!   real32Attribute(inevents,"RA_PNT") /= 0               .or. &
!   real32Attribute(inevents,"DEC_PNT") /= 0              .or. &
!   real32Attribute(inevents,"PA_PNT") /= 0               .or. &
    stringAttribute(inevents,"RADECSYS") /= "FK5"         .or. &        
    real32Attribute(inevents,"EQUINOX") /= 2000           .or. &
    stringAttribute(inevents,"OBSERVER") /= "Dr. Pino Paperino") &
    call error("wrongKeyword", &
              "Wrong value in standard primary header keyword")

   !-------------------------------------------------------------
   call message(AppMsg,Noisy, &
               "testing column existence in events extension...")
   !-------------------------------------------------------------
   eventstable = table(inevents,"EVENTS")
   nrows = numberOfRows(eventstable)

   if (.not.hasColumn(eventstable,"TIME ")   .or. &
       .not.hasColumn(eventstable,"RAWX")    .or. &
       .not.hasColumn(eventstable,"RAWY")    .or. &
       .not.hasColumn(eventstable,"DETX")    .or. &
       .not.hasColumn(eventstable,"DETY")    .or. & 
       .not.hasColumn(eventstable,"X")       .or. &
       .not.hasColumn(eventstable,"Y")       .or. & 
       .not.hasColumn(eventstable,"PHA")     .or. &
       .not.hasColumn(eventstable,"PI")      .or. &
       .not.hasColumn(eventstable,"PATTERN") .or. &
       .not.hasColumn(eventstable,"FLAG")    .or. &
       .not.hasColumn(eventstable,"PAT_ID")  .or. &
       .not.hasColumn(eventstable,"PAT_SEQ") .or. &
       .not.hasColumn(eventstable,"CCDNR"))       &
       call error("noEvColumn", &
                        "Events extension: column absent or incorrectly named")


   !----------------------------------------------------------------
   call message(AppMSg,Noisy, &
               "testing column existence in exposure extension...")
   !----------------------------------------------------------------
   do i = 7, 8
    write(extname,'(a,i2.2)') 'EXPOSU',i
  
    exptable = table(inevents,extname)

    if (.not.hasColumn(exptable,"TIME")       .or. &
!       .not.hasColumn(exptable,"TIMEDEL")    .or. &
        .not.hasAttribute(exptable,"TIMEDEL")    .or. &
        .not.hasColumn(exptable,"FRACEXP")) then
        write(errstr,*) 'Exposure extension ',extname, &
                        ': column absent or incorrectly named' 
        call error("noExposureColumn")
    endif
   enddo  

   !----------------------------------------------------------------
   call message(AppMSg,Noisy, &
               "testing column existence in bad pixel extension...")
   !----------------------------------------------------------------
   do i = 7, 8
    write(extname,'(a,i2.2)') 'BADPIX',i
  
    bpixtable = table(inevents,extname)

    if (.not.hasColumn(bpixtable,"RAWX")       .or. &
        .not.hasColumn(bpixtable,"RAWY")       .or. &
        .not.hasColumn(bpixtable,"TYPE")       .or. &
        .not.hasColumn(bpixtable,"YEXTENT")    .or. & 
        .not.hasColumn(bpixtable,"BADFLAG")) then
        write(errstr,*) 'Badpix extension ',extname, &
                        ': column absent or incorrectly named' 
        call error("noBadpixColumn")
    endif
   enddo    

   !----------------------
   call release(inevents)

   end
